@@ -1,8 +1,8 @@
-## DNA.R (2018-03-22)
+## DNA.R (2020-04-27)
 
 ##   Manipulations and Comparisons of DNA and AA Sequences
 
-## Copyright 2002-2018 Emmanuel Paradis, 2015 Klaus Schliep, 2017 Franz Krah
+## Copyright 2002-2020 Emmanuel Paradis, 2015 Klaus Schliep, 2017 Franz Krah
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -135,7 +135,6 @@ as.list.DNAbin <- function(x, ...)
 }
 
 rbind.DNAbin <- function(...)
-### works only with matrices for the moment
 {
     obj <- list(...)
     n <- length(obj)
@@ -146,9 +145,8 @@ rbind.DNAbin <- function(...)
     NC <- unlist(lapply(obj, ncol))
     if (length(unique(NC)) > 1)
         stop("matrices do not have the same number of columns.")
-    for (i in 1:n) class(obj[[i]]) <- NULL
-    for (i in 2:n) obj[[1]] <- rbind(obj[[1]], obj[[i]])
-    structure(obj[[1]], class = "DNAbin")
+    for (i in 1:n) class(obj[[i]]) <- NULL # safe but maybe not really needed
+    structure(do.call(rbind, obj), class = "DNAbin")
 }
 
 cbind.DNAbin <-
@@ -263,7 +261,14 @@ print.DNAbin <- function(x, printlen = 6, digits = 3, ...)
     if (nTot <= 1e7) {
         cat("Base composition:\n")
         print(round(base.freq(x), digits))
-    } else cat("More than 10 million nucleotides: not printing base composition\n")
+    } else {
+        cat("More than 10 million bases: not printing base composition.\n")
+    }
+    if (nTot > 1) {
+        k <- floor(log(nTot, 1000))
+        units <- c("bases", "kb", "Mb", "Gb", "Tb", "Pb", "Eb")
+        cat("(Total: ", round(nTot/1000^k, 2), " ", units[k + 1], ")\n", sep = "")
+    }
 }
 
 as.DNAbin <- function(x, ...) UseMethod("as.DNAbin")
@@ -376,15 +381,16 @@ Ftab <- function(x, y = NULL)
 
 GC.content <- function(x) sum(base.freq(x)[2:3])
 
-seg.sites <- function(x)
+seg.sites <- function(x, strict = FALSE, trailingGapsAsN = TRUE)
 {
     if (is.list(x)) x <- as.matrix(x)
     ## is.vector() returns FALSE because of the class,
     ## so we use a different test
     dx <- dim(x)
-    if (is.null(dx)) return(integer(0))
-    if (dx[1] == 1) return(integer(0))
-    ans <- .Call(SegSites, x)
+    if (is.null(dx)) return(integer())
+    if (dx[1] == 1) return(integer())
+    if (trailingGapsAsN) x <- latag2n(x)
+    ans <- .Call(SegSites, x, strict)
     which(as.logical(ans))
 }
 
@@ -498,7 +504,7 @@ image.DNAbin <-
         brks <- c(-0.5, brks)
     }
     yaxt <- if (show.labels) "n" else "s"
-    image.default(1:s, 1:n, t(y[n:1, ]), col = co, xlab = xlab,
+    image.default(1:s, 1:n, t(y[n:1, , drop = FALSE]), col = co, xlab = xlab,
                   ylab = ylab, yaxt = yaxt, breaks = brks, ...)
     if (show.labels)
         mtext(rownames(x), side = 2, line = 0.1, at = n:1,
@@ -524,7 +530,7 @@ image.DNAbin <-
     }
 }
 
-alview <- function(x, file = "", uppercase = TRUE)
+alview <- function(x, file = "", uppercase = TRUE, showpos = TRUE)
 {
     if (is.list(x)) x <- as.matrix(x)
     taxa <- formatC(labels(x), width = -1)
@@ -539,8 +545,21 @@ alview <- function(x, file = "", uppercase = TRUE)
     x <- apply(x, 1L, paste, collapse = "")
     if (uppercase) x <- toupper(x)
     res <- paste(taxa, x)
-    hdr <- formatC(s, width = nchar(res[1]))
-    cat(hdr, file = file, sep = "\n")
+    if ((is.logical(showpos) && showpos) || is.numeric(showpos)) {
+        if (is.logical(showpos)) {
+            pos <- 1:s
+            digits <- floor(log10(s)) + 1
+        } else {
+            pos <- showpos
+            digits <- floor(log10(max(pos))) + 1
+        }
+        hdr <- sprintf(paste0("%0", digits, "d"), pos)
+        hdr <- unlist(strsplit(hdr, ""))
+        dim(hdr) <- c(digits, length(pos))
+        hdr <- apply(hdr, 1, paste, collapse = "")
+        hdr <- formatC(hdr, width = nchar(res[1]))
+        cat(hdr, file = file, sep = "\n")
+    }
     cat(res, file = file, sep = "\n", append = TRUE)
 }
 
@@ -637,8 +656,8 @@ trans <- function(x, code = 1, codonstart = 1)
     f <- function(x, s, code)
         .C(trans_DNA2AA, x, as.integer(s), raw(s/3), as.integer(code),
            NAOK = TRUE)[[3]]
-    if (code > 2)
-        stop("only the standard and the vertebrate mitochondrial codes are available for now")
+    if (code > 6)
+        stop("only the genetic codes 1--6 are available for now")
 
     if (codonstart > 1) {
         del <- -(1:(codonstart - 1))
@@ -726,8 +745,11 @@ as.AAbin.character <- function(x, ...)
 
 labels.AAbin <- function(object, ...) labels.DNAbin(object, ...)
 
+## TO BE MOVED TO phangorn LATER
 if (getRversion() >= "2.15.1") utils::globalVariables("phyDat")
 as.phyDat.AAbin <- function(x, ...) phyDat(as.character(x), type = "AA")
+## \alias{as.phyDat.AAbin}
+## \method{as.phyDat}{AAbin}(x, \dots)
 
 dist.aa <- function(x, pairwise.deletion = FALSE, scaled = FALSE)
 {
@@ -1068,6 +1090,71 @@ c.AAbin <- function(..., recursive = FALSE)
   structure(NextMethod("c"), class = "AAbin")
 }
 
+rbind.AAbin <- function(...)
+{
+    obj <- list(...)
+    n <- length(obj)
+    if (n == 1) return(obj[[1]])
+    for (i in 1:n)
+        if (!is.matrix(obj[[1]]))
+            stop("the 'rbind' method for \"AAbin\" accepts only matrices")
+    NC <- unlist(lapply(obj, ncol))
+    if (length(unique(NC)) > 1)
+        stop("matrices do not have the same number of columns.")
+    for (i in 1:n) class(obj[[i]]) <- NULL # safe but maybe not really needed
+    structure(do.call(rbind, obj), class = "AAbin")
+}
+
+cbind.AAbin <-
+    function(..., check.names = TRUE, fill.with.Xs = FALSE,
+             quiet = FALSE)
+{
+    obj <- list(...)
+    n <- length(obj)
+    if (n == 1) return(obj[[1]])
+    for (i in 1:n)
+        if (!is.matrix(obj[[1]]))
+            stop("the 'cbind' method for \"AAbin\" accepts only matrices")
+    NR <- unlist(lapply(obj, nrow))
+    for (i in 1:n) class(obj[[i]]) <- NULL
+    if (check.names) {
+        NMS <- lapply(obj, rownames)
+        for (i in 1:n)
+            if (anyDuplicated(NMS[[i]]))
+                stop("Duplicated rownames in matrix ", i, ": see ?cbind.AAbin")
+        nms <- unlist(NMS)
+        if (fill.with.Xs) {
+            NC <- unlist(lapply(obj, ncol))
+            nms <- unique(nms)
+            ans <- matrix(charToRaw("X"), length(nms), sum(NC))
+            rownames(ans) <- nms
+            from <- 1
+            for (i in 1:n) {
+                to <- from + NC[i] - 1
+                k <- match(NMS[[i]], nms)
+                ans[k, from:to] <- obj[[i]]
+                from <- to + 1
+            }
+        } else {
+            tab <- table(nms)
+            ubi <- tab == n
+            nms <- names(tab)[which(ubi)]
+            ans <- obj[[1]][nms, , drop = FALSE]
+            for (i in 2:n)
+                ans <- cbind(ans, obj[[i]][nms, , drop = FALSE])
+            if (!quiet && !all(ubi))
+                warning("some rows were dropped.")
+        }
+    } else {
+        if (length(unique(NR)) > 1)
+            stop("matrices do not have the same number of rows.")
+        ans <- matrix(unlist(obj), NR)
+        rownames(ans) <- rownames(obj[[1]])
+    }
+    class(ans) <- "AAbin"
+    ans
+}
+
 as.AAbin.list <- function(x, ...)
 {
   obj <- lapply(x, as.AAbin)
@@ -1106,3 +1193,173 @@ as.matrix.AAbin <- function(x, ...)
   class(y) <- "AAbin"
   y
 }
+
+rDNAbin <- function(n, nrow, ncol, base.freq = rep(0.25, 4), prefix = "Ind_")
+{
+    foo <- function(n, prob) {
+        vec <- as.raw(._bs_[1:4])
+        vec[sample.int(4L, n, TRUE, prob, FALSE)]
+    }
+    base.freq <- if (all(base.freq == 0.25)) NULL else base.freq[c(1, 3, 2, 4)]
+    if (missing(n)) {
+        if (missing(nrow) && missing(ncol))
+            stop("nrow and ncol should be given if n is missing")
+        res <- foo(nrow * ncol, base.freq)
+        dim(res) <- c(nrow, ncol)
+        rownames(res) <- paste0(prefix, 1:nrow)
+    } else {
+        res <- lapply(n, foo, prob = base.freq)
+        names(res) <- paste0(prefix, seq_along(n))
+    }
+    class(res) <- "DNAbin"
+    res
+}
+
+dnds <- function(x, code = 1, codonstart = 1, quiet = FALSE)
+{
+    if (code > 2)
+        stop("only the standard (code=1) and the vertebrate mitochondrial (code=2) codes are available for now")
+    if (is.list(x)) x <- as.matrix(x)
+    n <- nrow(x)
+    if (nrow(unique.matrix(x)) != n) stop("sequences are not unique")
+    if (codonstart > 1) {
+        del <- -(1:(codonstart - 1))
+        x <- x[, del]
+    }
+    p <- ncol(x)
+    rest <- p %% 3
+    if (rest != 0) {
+        p <- p - rest
+        x <- x[, 1:p]
+        msg <- paste("sequence length not a multiple of 3:",
+                     rest, ifelse(rest == 1,  "nucleotide", "nucleotides"),
+                     "dropped")
+        warning(msg)
+    }
+
+    class(x) <- NULL
+
+    abin <- as.raw(0x88)
+    gbin <- as.raw(0x48)
+    cbin <- as.raw(0x28)
+    tbin <- as.raw(0x18)
+
+    pos1 <- seq(1, by = 3, length.out = p/3)
+    POS1 <- POS2 <- POS3 <- logical(p)
+    POS1[pos1] <- POS2[pos1 + 1L] <- POS3[pos1 + 2L] <- TRUE
+
+    Lcat <- matrix(0L, n, p)
+    Lcat[, POS3] <- 4L # most sites at 3rd positions are fourfold degenerate
+
+    m12 <- -c(1, 2)
+
+    if (code == 1) {
+        for (i in 1:n) {
+            z <- x[i, , drop = TRUE]
+            isA <- z == abin
+            isG <- z == gbin
+            isC <- z == cbin
+            isT <- z == tbin
+            ## 1/ are some bases at 1st positions twofold degenerate?
+            s2 <- isT & POS2
+            if (any(s2)) {
+                s3 <- c(FALSE, s2[-p]) & ((isA | isG) & POS3)
+                if (any(s3)) {
+                    s1 <- ((isT | isC) & POS1) & c(s3[m12], FALSE, FALSE)
+                    if (any(s1)) Lcat[i, which(s1)] <- 2L
+                }
+            }
+            s2 <- isG & POS2
+            if (any(s2)) {
+                s3 <- c(FALSE, s2[-p]) & ((isA | isG) & POS3)
+                if (any(s3)) {
+                    s1 <- ((isA | isC) & POS1) & c(s3[m12], FALSE, FALSE)
+                    if (any(s1)) Lcat[i, which(s1)] <- 2L
+                }
+            }
+            ## 2/ all bases at 2nd positions are nondegenerate: no need to do anything
+            ## 3/ find the bases at 3rd positions that are twofold degenerate
+            s2 <- isA & POS2
+            if (any(s2)) Lcat[i, which(s2) + 1L] <- 2L
+            s2 <- isG & POS2
+            if (any(s2)) {
+                s1 <- ((isT | isA) & POS1) & c(s2[-p], FALSE)
+                if (any(s1)) Lcat[i, which(s1) + 2L] <- 2L
+            }
+        }
+    } else { # if (code == 2)
+        for (i in 1:n) {
+            z <- x[i, , drop = TRUE]
+            isA <- z == abin
+            isG <- z == gbin
+            isC <- z == cbin
+            isT <- z == tbin
+            ## 1/ are some bases at 1st positions twofold degenerate?
+            s2 <- isT & POS2
+            if (any(s2)) {
+                s3 <- c(FALSE, s2[-p]) & ((isA | isG) & POS3)
+                if (any(s3)) {
+                    s1 <- ((isT | isC) & POS1) & c(s3[m12], FALSE, FALSE)
+                    if (any(s1)) Lcat[i, which(s1)] <- 2L
+                }
+            }
+            ## 2/ all bases at 2nd positions are nondegenerate: no need to do anything
+            ## 3/ find the bases at 3rd positions that twofold/fourfold degenerate
+            s1 <- (isA | isT) & POS1
+            if (any(s1)) {
+                s2 <- c(FALSE, s1[-p]) & ((isT | isA | isG) & POS2)
+                if (any(s2)) Lcat[i, which(s2) + 1L] <- 2L
+            }
+            s1 <- (isC | isG) & POS1
+            if (any(s1)) {
+                s2 <- c(FALSE, s1[-p]) & (isA & POS2)
+                if (any(s2)) Lcat[i, which(s2) + 1L] <- 2L
+            }
+        }
+    }
+
+    deg <- c(0, 2, 4) # the 3 levels of degeneracy
+    nout <- n*(n - 1)/2
+    res <- numeric(nout)
+    k <- 1L
+    for (i in 1:(n - 1)) {
+        for (j in (i + 1):n) {
+            if (!quiet) cat("\r", round(100*k/nout), "%")
+            z <- x[c(i, j), ]
+            Lavg <- (Lcat[i, ] + Lcat[j, ])/2
+            ii <- lapply(deg, function(x) which(x == Lavg))
+            L <- lengths(ii)
+            S <- lapply(ii, function(id) dist.dna(z[, id], "TS"))
+            V <- lapply(ii, function(id) dist.dna(z[, id], "TV"))
+            P <- unlist(S, use.names = FALSE)/L
+            Q <- unlist(V, use.names = FALSE)/L
+            a <- 1/(1 - 2*P - Q)
+            b <- 1/(1 - 2*Q)
+            c <- (a - b)/2
+            A <- log(a)/2 - log(b)/4
+            B <- log(b)/2
+            dS <- (L[2]*A[2] + L[3]*A[3])/sum(L[2:3]) + B[3]
+            dN <- A[1] + (L[1]*B[1] + L[2]*B[2])/sum(L[1:2])
+            res[k] <- dN/dS
+            k <- k + 1L
+        }
+    }
+    if (!quiet) cat("... done\n")
+    attr(res, "Size") <- n
+    attr(res, "Labels") <- rownames(x)
+    attr(res, "Diag") <- attr(res, "Upper") <- FALSE
+    attr(res, "call") <- match.call()
+    attr(res, "method") <- "dNdS (Li 1993)"
+    class(res) <- "dist"
+    res
+}
+
+latag2n <- function(x) {
+    if (is.list(x)) x <- as.matrix(x)
+    dx <- dim(x)
+    clx <- class(x)
+    res <- .Call(leading_trailing_gaps_to_N, x)
+    class(res) <- clx
+    dim(res) <- dx
+    res
+}
